#
# operator-on-uniform.R, 15 Apr 23

library("colorspace")


par(bty="l")
par(las=1)
par(pch="+")
pal_col=rainbow(5)


plot(-1, type="n",
	xaxs="i", yaxs="i",
	xlim=c(0, 5), ylim=c(0, 1),
	xlab="Result", ylab="Probability density")

x01=seq(0, 1, 0.01)
x12=seq(1, 2, 0.01)
x23=seq(2, 3, 0.01)
x34=seq(3, 4, 0.01)
x45=seq(4, 5, 0.01)

# n=1
lines(c(0, 1), c(1, 1), col=pal_col[1])

# n=2
lines(x01, x01, col=pal_col[2])
lines(x12, 2-x12, col=pal_col[2])

# n=3
lines(x01, 0.5*x01^2, col=pal_col[3])
lines(x12, 0.5*(-2*x12^2+6*x12-3), col=pal_col[3])
lines(x23, 0.5*(3-x23)^2, col=pal_col[3])

# n=4
lines(x01, (1/6)*x01^3, col=pal_col[4])
lines(x12, (1/6)*(-3*x12^3+12*x12^2-12*x12+4), col=pal_col[4])
lines(x23, (1/6)*(3*x23^3-24*x23^2+60*x23-44), col=pal_col[4])
lines(x34, (1/6)*(4-x34)^3, col=pal_col[4])

# n=6
lines(x01, (1/24)*x01^4, col=pal_col[5])
lines(x12, (1/24)*(-4*x12^4+20*x12^3-30*x12^2+20*x12-5), col=pal_col[5])
lines(x23, (1/24)*(6*x23^4-60*x23^3+210*x23^2-300*x23+155), col=pal_col[5])
lines(x34, (1/24)*(-4*x34^4+60*x34^3-330*x34^2+780*x34-655), col=pal_col[5])
lines(x45, (1/24)*(5-x45)^4, col=pal_col[5])

legend("topright", legend=paste("N= ", 1:5),
                        fill=pal_col, border="white", bty="n")

dev.copy(dev=png, file="add-unif-distrib.png")
dev.off()


par(mfcol=c(2, 1))

# Distribution of bitwise AND of two uniform distributions
ba=bitwAnd(runif(1e6, 0, 32767), runif(1e6, 0, 32767))
plot(count(ba), log="y", col="salmon", main="And",
	xaxs="i",
	xlab="Result", ylab="Occurrences")

# Distribution of bitwise OR of two uniform distributions
bo=bitwOr(runif(1e6, 0, 32767), runif(1e6, 0, 32767))
plot(count(bo), log="y", col="salmon", main="Or",
	xaxs="i",
	xlab="Result", ylab="Occurrences")

dev.copy(dev=png, file="and-or-unif-distrib.png")
dev.off()

par(mfcol=c(1, 1))

# Distribution of multiplication of two uniform distributions
mul=trunc(runif(1e6, 0, 100)) * trunc(runif(1e6, 0, 100))

plot(count(mul), log="y", col="salmon",
	xaxs="i",
	xlim=c(-50, 10000),
	xlab="Result", ylab="Occurrences")

dev.copy(dev=png, file="mul-unif-distrib.png")
dev.off()

cmul=count(mul)
c200=subset(cmul, (freq > 170) & (freq < 240))
head(c200, n=10)

c300=subset(cmul, (freq > 360) & (freq < 440))
head(c300, n=10)

c1200=subset(cmul, (freq > 1160) & (freq < 1240))
head(c1200, n=10)

# Distribution of division of two uniform distributions
div=trunc(trunc(runif(1e6, 0, 100)) / trunc(runif(1e6, 1, 100)))
plot(count(div), log="xy")

cdiv=count(div)
cd_mod=glm(log(freq) ~ log(x), data=cdiv, subset=(x > 0))
summary(cd_mod)

xbounds=1:100
pred=predict(cd_mod, newdata=data.frame(x=xbounds))
lines(xbounds, exp(pred), col=pal_col[1])


# List of integer pairs for values in range 1:1e5
int_pairs=read.csv("b038548.txt")
rep_occur=rep.int(int_pairs$value, times=int_pairs$pairs)

# replicate that probability of equality for 1e5 unique values is 1 in 1e5
lhs=sample(1:1e5, size=1e7, replace=TRUE)
rhs=sample(1:1e5, size=1e7, replace=TRUE)

num_eq=length(which(lhs == rhs))
num_eq/1e7
# [1] 9.8e-06

lhs=sample(rep_occur, size=1e7, replace=TRUE)
rhs=sample(rep_occur, size=1e7, replace=TRUE)

num_eq=length(which(lhs == rhs))
num_eq/1e7
# [1] 0.0001757

